;
;	xscrn.com	... switch screen to lcd | crt  ver 1.1
;	    xscrn [m11]    m:"shift screen to right by 02...3f" and "stay"
;		use "Ctrl/Alt + F7" keys for switching

;		start	2023.11.27
;		final	2023.11.30

;				by GarakutaEn
;--------------------------------------
code	segment
	assume	cs:code,es:code,ss:code
	org	100h
;--------------------------------------
;ascii codes
bel	equ	07h	;BEL
bs	equ	08h	;back space code
cr	equ	0dh	;carriage return
lf	equ	0ah	;line feed
esk	equ	1bh	;ESC key
ff	equ	0ch	;form feed
cf7	equ	64h	;ctrl+f7 on rom bios
af7	equ	6eh	;alt+f7 on rom bios

;dos int services
rdky	equ	01h	;wait key = al
wrch	equ	02h	;write a char = dl
drky	equ	07h	;direct key check without echo
rdch	equ	08h	;read a char = al
wrln	equ	09h	;write a line = ds:dx
rdln	equ	0ah	;read line = ds:dx
gdrv	equ	19h	;bootstrap loader

svct	equ	25h	;ds:dx
gvct	equ	35h	;es:bx

srtn	equ	31h	;return, but keep process dx / exe
nrtn	equ	4ch	;normal return to dos

gtmem	equ	48h	;bx
rlmem	equ	49h	;es paragrph

mkfl	equ	3ch	;make a file = ds:dx, cx
opfl	equ	3dh	;open the file = ds:dx, al
clfl	equ	3eh	;close the file = bx
rdfl	equ	3fh	;read from the file = ds:dx, bx, cx
wrfl	equ	40h	;write to the file = ds:dx, bx, cx

;rom int services
inky	equ	00h
ckky	equ	01h	;if key, zero_flag = 0 ; al = ASCII, ah = ID/code
ttyo	equ	0eh	;set to al
kstt	equ	11h	;key status

;rom bios int
kint	equ	09h	;keybrd int
vint	equ	10h	;video int
dint	equ	13h	;disk int
tint	equ	62h	;90h	;temp int / free to use

;XR reg of ct65535
xidx	equ	3d6h
xval	equ	3d7h

;--------------------------------
;macro
dos	macro	cmd
	mov	ah,cmd
	int	21h
	endm
;
pusha	macro
	db	60h
	endm
popa	macro
	db	61h
	endm
;
;video / rom bios
romv	macro	cmd
	mov	ah,cmd
	int	10h
	endm
;disk / rom bios
romd	macro	cmd
	mov	ah,cmd
	int	13h
	endm
;key / rom bios
romk	macro	cmd
	mov	ah,cmd
	int	16h
	endm
;---------------------------------
start:	cld
	mov	ax,cs
	mov	es,ax
	mov	ss,ax
	lea	sp,stack
init:	mov	si,80h
	cmp	byte ptr [si],0
	je	inby		;no shift, no stay
ini0:	inc	si
	mov	al,[si]
	cmp	al,cr
	je	inby
	and	al,0dfh
	cmp	al,'M'
	je	inst
ini1:	jmp	ini0
ini2:	jmp	ini0
;release
inby:	mov	al,kint
	dos	gvct
	lea	dx,fmsg		;now free
	cmp	bx,offset kychk	;check existing "new" ?
	jne	inb0		;no need to restore ?
	cli
	call	rsvct		;restore org_vect
	sti
	push	cs
	pop	ds
	lea	dx,cmsg		;now clear
inb0:	dos	wrln
iniz:	xor	al,al
	dos	nrtn		;normal return

;stay processing
inst:	mov	ax,[si+1]
	call	cvd2b		;cnvt ascii to bin
	cmp	al,02h
	jge	ins0
	mov	al,02h
ins0:	and	al,3fh		;set limit
	mov	cs:[xr56+1],al

	mov	al,kint		;09h
	dos	gvct		;get int_key vector
	cmp	bx,offset kychk	;exist already ?
	jne	ins1

	call	stscr		;set screen
	xor	ax,ax
	dos	nrtn		;return

ins1:	call	cgvct		;change vector
	lea	dx,smsg
	dos	wrln
	call	stscr

	mov	ax,[2ch]
	mov	es,ax
	dos	rlmem		;release memory
	lea	dx,stack+2
	dos	srtn		;stay and return (dos v3.0,...)
;--------------------------------------
;chng to new vector for int_key
cgvct:	cli
	mov	dx,bx		;es:bx
	push	es
	pop	ds
	mov	al,tint		;62h, 90h copy int_key to int_temp
	dos	svct
	lea	dx,kychk	;set dx
	push	cs
	pop	ds		;ds:dx
	mov	al,kint		;09h
	dos	svct		;set temp vector
	sti
	ret
;++++++++++++++++++++++++++++++++++++++
;++ entering point of int_kint ++
;check key for xchg scrn
kychk:	int	tint		;62h, 90h
	cli
	pusha
	romk	kstt		;check status
	jz	kyche
	and	al,al
	jnz	kyche
	mov	al,ah
	cmp	al,cf7
	jne	kych1
	call	stscr
	jmp	kyche
kych1:	cmp	ah,af7
	jne	kyche
	call	stscr
kyche:	popa
	sti
	iret
;++++++++++++++++++++++++++++++++++++++
;restore int vector
rsvct:	cli
	mov	al,tint		;90h
	dos	gvct		;es:bx
	mov	dx,bx
	push	es
	pop	ds
	mov	al,kint		;09h
	dos	svct		;ds:dx
	xor	dx,dx		;keep clean !
	mov	ds,dx
	mov	al,tint
	dos	svct
	ret
;--------------------------------------
;set screen lcd or crt by cond. of xr1bh
stscr:	mov	al,51h	;1bh		;good XR for mark
	mov	dx,xidx		;3d6h
	out	dx,al
	mov	dx,xval		;3d7h
	in	al,dx
stlcd:	lea	si,tftbl	;TFT_LCD table
	cmp	al,0c4h	;0a0h		;normal value of Flt_Pnel
	jne	wtrgs
stcrt:	lea	si,crtbl	;CRT/Ext table
;write all reg.s
wtrgs:	mov	al,cs:[si]
	cmp	al,0ffh
	jne	wtrg0
	ret
wtrg0:	mov	dx,xidx		;3d6h
	out	dx,al
	mov	al,cs:[si+1]
	mov	dx,xval		;3d7h
	out	dx,al
	add	si,2
	jmp	wtrgs

;++++++++++++++++++++++++++++++++++++++
;parameter table
tftbl:	db	06h,0c0h	;No color reduction
	db	19h,00h		;Alt Hsync Start(CR04)
	db	1ah,02h		;Alt Hsync End(CR05)
	db	1bh,7fh		;Alt H Total(CR00)
	db	1ch,70h		;Panel Width
	db	4fh,0c6h	;LP/M change to DE
	db	50h,00h		;Clock divide, Dither, FRC
	db	51h,0c4h	;change to LCD on 4h !!!
	db	52h,42h		;for 'Panel On'
	db	53h,00h		;for FRC
	db	54h,0c8h	;Negative Polarity
	db	55h,0e5h	;H Auto Centering
xr56:	db	56h,10h		;H Left Border / Centering
	db	6ch,1ch		;output buffer x2
	db	73h,0ah		;DPMS (CRT killer)
	db	0ffh,0ffh
;table end
;register table of the original data
crtbl	db	06h,0c0h
	db	19h,01h
	db	1ah,19h
	db	1bh,59h
	db	1ch,4fh
	db	4fh,04h
	db	50h,15h
	db	51h,0c0h	;63h?	;Change to CRT !!!
;	db	52h,42h
	db	53h,00h
	db	54h,3ah
	db	55h,0e5h
	db	56h,00h
	db	73h,05h
	db	0ffh,0ffh
;table end
;++++++++++++++++++++++++++++++++++++++
;std sub;
;display space
dssp:	push	dx
	mov	dl,' '
	dos	wrch
	pop	dx
	ret
;--------------------------------------
;put cr & lf
crlf:	push	dx
	mov	dl,cr
	dos	wrch
	mov	dl,lf
	dos	wrch
	pop	dx
	ret
;--------------------------------------
;display 2 hex_ascii on ax(ah+al)
ds2hx:	push	dx
	push	ax
	mov	al,ah
	dos	wrch
	pop	ax		;use al
	pop	dx
;display 1 hex_ascii on al
ds1hx:	push	dx
	mov	dl,al
	dos	wrch
	pop	dx
	ret
;--------------------------------------
;display 2 binary on ax with 4 hex_ascii
ds2bx:	push	ax
	mov	al,ah
	call	cvb2h
	call	ds2hx
	pop	ax
;display 1 binary on al with 2 hex_ascii
ds1bx:	call	cvb2h
	call	ds2hx
	ret
;--------------------------------------
;cnv double/2 hex to bin(1 byte)
cvd2b:	call	cvh2b
	xchg	ah,al
	call	cvh2b
	and	al,0fh
	shl	ah,1
	shl	ah,1
	shl	ah,1
	shl	ah,1
	and	ah,0f0h
	add	al,ah
	ret
;--------------------------------------
;convert hex_ascii to bin(half byte)
cvh2b:	sub	al,'0'
	jc	cvh2e
	cmp	al,0ah
	jc	cvh21
	and	al,0dfh
	sub	al,7
	cmp	al,0fh
cvh21:	cmc
cvh2e:	ret
;--------------------------------------
;convert a binary(al) to 2 hex_ascii(ax)
cvb2h:	mov	ah,al
	and	al,0fh
	add	al,'0'
	cmp	al,'9'+1
	jb	cvb21
	add	al,7
cvb21:	shr	ah,1
	shr	ah,1
	shr	ah,1
	shr	ah,1
	add	ah,'0'
	cmp	ah,'9'+1
	jb	cvb2e
	add	ah,7
cvb2e:	clc
	ret
;end of sub
;--------------------------------------
hmsg	db	'switch screen to lcd | crt',cr,lf,'$'
emsg	db	'error !',cr,lf,'$'
fmsg	db	' ... ',cr,lf,'$'
smsg	db	'stay...',cr,lf,'$'
cmsg	db	'...clear',cr,lf,'$'
stk	dw	128 dup (?)
stack	equ	$
;--------------------------------------
code	ends
	end	start
;--------------------------------------